//
//  InAppPurchaseManager.m
//  InAppPurchaseXcode
//
//  Created by Tetsuya Shiraishi on 12/05/23.
//  Copyright (c) 2012年 MUSHIKAGO.INC. All rights reserved.
//

#import "InAppPurchaseManager.h"
//#define kInAppPurchaseProUpgradeProductId @"com.mushikago.InAppPurchaseXcode.unLock"
//#define kAppProductId @"com.mushikago.InAppPurchaseXcode"

@implementation InAppPurchaseManager

@synthesize myContext; //step5

// 「3. ペイメント処理を進めてよいか判断します。」部分
- (BOOL)canMakePurchases
{
    if([SKPaymentQueue canMakePayments])
    {
        return TRUE;
    }else {
        return FALSE;
    }
}

// 「4. プロダクトに関する情報を取得します。」部分
- (void)requestProductData:(NSString *) pID
{
    myProductRequest= [[SKProductsRequest alloc]
                       initWithProductIdentifiers: [NSSet setWithObject: pID]];
    myProductRequest.delegate = self;
    [myProductRequest start];
}

- (void)productsRequest:(SKProductsRequest *)request
     didReceiveResponse:(SKProductsResponse *)response
{
    
    if (response == nil) {
        [self showAlert:@"responseなし"];
        return;
    }
    
    // 確認できなかったidentifierをログに記録
    for (NSString *identifier in response.invalidProductIdentifiers) {
        NSLog(@"invalid product identifier: %@", identifier);
    }
    
    //NSArray *products = response.products; // UIを埋める
    //NSString *message = [NSString stringWithFormat:@"配列は%d個です。", [products count]];
    
    //NSString *message = @"何か戻りました。";
    //NSLog(@"配列は%d個です。", [products count]);
    //[self showAlert:message];
    
    //[myProductRequest autorelease];
    
    //Observer
    //InAppPurchaseStoreObserver *observer = [[InAppPurchaseStoreObserver alloc]init];
    //[[SKPaymentQueue defaultQueue] addTransactionObserver:observer];
    
    //  「6. トランザクションオブザーバをペイメントキューに登録します。」部分
    // （※本当は起動時の方がいい）
    [[SKPaymentQueue defaultQueue] addTransactionObserver:self];
    
    for (SKProduct *product in response.products ) {
        NSLog(@"valid product identifier: %@", product.productIdentifier);
        
        NSLog(@"%@",[product productIdentifier]);
        NSLog(@"%@",[product localizedTitle]);
        NSLog(@"%@",[product localizedDescription]);
        
        //SKPayment *payment = [SKPayment paymentWithProduct:product];
        //[[SKPaymentQueue defaultQueue] addPayment:payment];
        
        // 「11. ・・・ユーザ がStore内のアイテムを選択したら、ペイメントオブジェクトを作成し、ペイメントキューに追加 します。」部分
        SKPayment *payment = [SKPayment paymentWithProduct:product];
        [[SKPaymentQueue defaultQueue] addPayment:payment];
    }
    
}

// 「7. MyStoreObserverにpaymentQueue:updatedTransactions:メソッドを実装します。」部分
- (void)paymentQueue:(SKPaymentQueue *)queue
 updatedTransactions:(NSArray *)transactions
{
    
    for (SKPaymentTransaction *transaction in transactions)
    {
        const uint8_t* msg1; //step5
        const uint8_t* msg2 = (uint8_t *)[transaction.payment.productIdentifier UTF8String]; //step5
        
        switch (transaction.transactionState)
        {
            case SKPaymentTransactionStatePurchasing:
                // 購入処理中。基本何もしなくてよい。処理中であることがわかるようにインジケータをだすなど。
                break;
            case SKPaymentTransactionStatePurchased:
                // 購入処理成功
                msg1 = (const uint8_t*)"Purchased"; //step5
                FREDispatchStatusEventAsync(myContext,msg1,msg2); //step5
                
                [self showAlert:@"購入処理成功！"];
                NSLog(@"購入処理成功！");
                NSLog(@"transaction : %@",transaction);
                NSLog(@"transaction.payment.productIdentifier : %@",transaction.payment.productIdentifier);
                [queue finishTransaction: transaction];
                break;
            case SKPaymentTransactionStateFailed:
                msg1 = (const uint8_t*)"Failed"; //step5
                FREDispatchStatusEventAsync(myContext,msg1,msg2); //step5
                
                // 購入処理失敗。ユーザが購入処理をキャンセルした場合もここ
                if (transaction.error.code != SKErrorPaymentCancelled)
                {
                    // 必要に応じてここでエラーを表示する
                    [self showAlert:[transaction.error localizedDescription]];
                    NSLog(@"購入処理失敗。");
                    NSLog(@"error : %@",[transaction.error localizedDescription]);
                }
                [queue finishTransaction:transaction];
                break;
            case SKPaymentTransactionStateRestored:
                //リストア処理開始
                msg1 = (const uint8_t*)"Restored"; //step5
                FREDispatchStatusEventAsync(myContext,msg1,msg2); //step5
                
                [self showAlert:@"リストア処理開始"];
                NSLog(@"リストア処理");
                NSLog(@"transaction : %@",transaction);
                NSLog(@"transaction.originalTransaction.payment.productIdentifier : %@",transaction.originalTransaction.payment.productIdentifier);
                [queue finishTransaction:transaction];
            default:
                break;
        } }
}

//Alert
- (void)showAlert:(NSString *)msg
{
    UIAlertView *alert = [ [UIAlertView alloc] initWithTitle:@"Message"
                                                     message:msg
                                                    delegate:self
                                           cancelButtonTitle:@"OK"
                                           otherButtonTitles:nil];
    [alert show];
}
@end